package org.javacoo.crawler.frontier;

import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.commons.lang.StringUtils;
import org.apache.http.client.ClientProtocolException;
import org.apache.http.client.methods.HttpGet;

import org.javacoo.crawler.CrawlController;
import org.javacoo.crawler.constants.Constants;
import org.javacoo.crawler.data.Task;
import org.javacoo.crawler.data.TaskQueue;
import org.javacoo.crawler.data.UrlQueue;
import org.javacoo.crawler.util.HttpGetHandler;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
/**
 * 边界控制器  接口 实现类
 * 主要是加载爬行种子链接并根据加载的种子链接初始化任务队列，以备线程控制器（ProcessorThreadPool）开启的任务执行线程（ProcessorThread）使用
 * @author javacoo
 * @since 2011-11-09
 */
public class DefaultFrontier implements Frontier{
	private Logger log = LoggerFactory.getLogger(DefaultFrontier.class);
	/**爬虫控制器*/
	private CrawlController controller;
	/**计划UrlQueue对象*/
	private UrlQueue planUrlQueue = new UrlQueue();
	/**任务对象*/
	private TaskQueue taskQueue = new TaskQueue();
	/**任务总数*/
    private transient int taskSize;
	public void finished(Task task) {
		taskQueue.addExecTask(task);
		log.info("已经完成任务："+taskQueue.getExecTaskNum()+"个");
	}
	/**
	 * 初始化
	 * <li>加载爬行种子链接</li>
	 * <li>初始化任务队列</li>
	 * @param c 控制器对象
	 */
	public void initialize(CrawlController c) {
        this.controller = c;
        loadSeeds();
        initTask();
        this.taskSize = taskQueue.getUnExecTaskNum();
	}
    /**
     * 初始化任务队列
     */
	private void initTask() {
		log.info("=========初始化任务队列=========");
		Map<String,String> urlMap = null;
		int planNum = 1;
		while(!planIsEmpty()) {
			urlMap = getPlan();
			populateTaskQueue(urlMap,planNum);
			planNum++;
		}
		log.info("=========初始化任务队列结束,任务总数：========="+taskQueue.getUnExecTaskNum()+"个");
	}
	/**
	 * 组装任务队列
	 * @param map
	 */
	private void populateTaskQueue(Map<String,String> map,Integer planNum){
		log.info("=========开始组装任务队列=========");
		HttpGet httpGet = null;
		try {
			String parentUrl = map.get(Constants.LINK_KEY).trim();
			log.info("=========原始URL========="+parentUrl);
			httpGet = HttpGetHandler.getHttpGet(new URI(parentUrl));
			String html = this.controller.getHttpClient().execute(httpGet, this.controller.getHandler());
			log.info("=========连接原始html========="+html);
			Task task = null;
			Map<String,String> resMap = new HashMap<String, String>();
			int taskNum = 1;
			
			List<Map<String,String>> urlAndTitleList = this.controller.getParseHtmlTool().getUrlAndTitleMap(html,this.controller.getCrawlScope().getSavePath());
			if(this.controller.getCrawlScope().isAllowRepeat()){
				//从采集历史表中检查是否已经采集过
				for(Map<String,String> planMap : urlAndTitleList){
					if(this.controller.getCrawlScope().getCrawlerPersistent().check(true,planMap.get(Constants.TITLE_KEY))){
						urlAndTitleList.remove(planMap);
					}
				}
			}
			for(Map<String,String> planMap : urlAndTitleList){
				task = new Task();
				task.setPlanNum(planNum);
				task.setTaskNum(taskNum);
				task.setCurrTaskTotalNum(urlAndTitleList.size());
				task.setParentUrl(parentUrl);
				resMap = new HashMap<String, String>();
				task.setController(this.controller);
				task.setUrl(planMap.get(Constants.LINK_KEY));
				task.getContentBean().setTitle(StringUtils.trim(planMap.get(Constants.TITLE_KEY)));
				task.getContentBean().setAcquId(this.controller.getCrawlScope().getId());
				if(this.controller.getCrawlScope().isExtractContentRes()){
					if(StringUtils.isNotEmpty(planMap.get(Constants.NEW_IMAGE_KEY))){
						task.getContentBean().setTitleImg(planMap.get(Constants.NEW_IMAGE_KEY));
						resMap.put(planMap.get(Constants.NEW_IMAGE_KEY), planMap.get(Constants.ORIGN_IMAGE_KEY));
					}
				}else if(StringUtils.isNotEmpty(planMap.get(Constants.ORIGN_IMAGE_KEY))){
					String tempOrginImgUrl = planMap.get(Constants.ORIGN_IMAGE_KEY);
					if(!tempOrginImgUrl.contains("http://")){
						tempOrginImgUrl = this.controller.getCrawlScope().getRepairImageUrl()+tempOrginImgUrl;
					}
					task.getContentBean().setTitleImg(tempOrginImgUrl);
					resMap.put(tempOrginImgUrl,planMap.get(Constants.NEW_IMAGE_KEY));
				}
				task.getContentBean().getResMap().put(Constants.RES_IMAGES_MAP_KEY,resMap);
				addTask(task);
				taskNum++;
			}
		} catch (URISyntaxException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (ClientProtocolException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}finally{
			if(null != httpGet){
				httpGet.abort();
			}
		}
	}
    /**
     * 加载种子链接
     */
	private void loadSeeds() {
		log.info("=========开始加载种子链接=========");
		List<String> plans = this.controller.getCrawlScope().getSeeds();
		Map<String,String> planMap = null;
		for (int i = 0; i < plans.size(); i++) {
			planMap = new HashMap<String,String>();
			planMap.put(Constants.LINK_KEY, plans.get(i));
			planMap.put(Constants.TITLE_KEY, "");
			addPlan(planMap);
		}
		log.info("=========加载种子链接结束,共加载种子数：========="+planUrlQueue.getUnVisitedUrlNum()+"个");
	}
    /**
     * 判断当前边界控制器任务队列是否为空
     */
	public boolean isEmpty() {
		return taskIsEmpty();
	}
    /**
     * 取得下一个任务
     */
	public Task next() {
		return getTask();
	}
	
	/**
	 * 连接和标题map对象入队列
	 * @param map 连接和标题map对象
	 */
	private void addPlan(Map<String,String> map){
		planUrlQueue.addUnVisitedUrl(map);
	}
	/**
	 * 连接和标题map对象出队列
	 * @param urlQueue 当前线程的队列
	 * @return 连接和标题map对象
	 */
	private Map<String,String> getPlan(){
		return planUrlQueue.unVisitedUrlDeQueue();
	}
	/**
	 *  判断当前对象是否为空
	 * @param urlQueue 当前线程的队列
	 * @return true/flase
	 */
	private boolean planIsEmpty(){
		return planUrlQueue.isEmpty();
	}
	/**
	 * 任务对象入队列
	 * @param map 任务对象
	 */
	private void addTask(Task task){
		taskQueue.addUnExecTask(task);
	}
	/**
	 * 任务对象出队列
	 * @param urlQueue 当前线程的队列
	 * @return 任务对象
	 */
	private Task getTask(){
		return taskQueue.unExecTaskDeQueue();
	}
	/**
	 *  判断当前对象是否为空
	 * @param urlQueue 当前线程的队列
	 * @return true/flase
	 */
	private boolean taskIsEmpty(){
		return taskQueue.isEmpty();
	}
	/**
	 * 取得任务总数
	 * @return 任务总数
	 */
	public int getTaskSize() {
		return taskSize;
	}
	/**
	 * 取得已执行任务总数
	 * @return 已执行任务总数
	 */
	public int getExecTaskNum() {
		return taskQueue.getExecTaskNum();
	}
	/**
	 * 取得未执行任务总数
	 * @return 未执行任务总数
	 */
	public int getUnExecTaskNum() {
		return taskQueue.getUnExecTaskNum();
	}
	/**
	 * 销毁对象
	 */
	public void destory(){
		taskQueue.clear();
		planUrlQueue.clear();
		controller = null;
		taskSize = 0;
	}
	

}
