package org.javacoo.crawler.processor.extractor;


import java.net.URI;
import java.util.List;

import org.apache.http.client.methods.HttpGet;
import org.javacoo.crawler.constants.Constants;
import org.javacoo.crawler.data.SimpleUrlQueue;
import org.javacoo.crawler.data.Task;
import org.javacoo.crawler.util.HttpGetHandler;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

/**
 * 任务处理器接口-抽取分页内容实现类
 * <li>抽取链 ： 当提分页内容 </li>
 * @author javacoo
 * @since 2012-02-09
 */
public class ExtractorPaginationProcessor extends Extractor{
	/**存放SimpleUrlQueue的ThreadLocal对象*/ 
	private static ThreadLocal<SimpleUrlQueue> pageUrlQueueThreadLocal = new ThreadLocal<SimpleUrlQueue>(); 
	
	public ExtractorPaginationProcessor() {
		super();
		setDefaultNextProcessor(new ExtractorContentResourceProcessor());
	}

	@Override
	protected void extract(Task task) {
		initProcessor(task);
		processorHTML(task);
	}
	/**
	 * 抽取第一页内容，得到内容分页
	 */
	private void initProcessor(Task task){
		log.info("=========抽取第一页内容,得到内容分页连接,加入到内容分页队列中=========");
		//第一页已经访问
		getUrlQueue().addVisitedUrl(task.getUrl().trim());
		populatePaginationList(task,task.getOrginHtml());
	}

	/**
	 * 抽取原始HTML中分页链接,加入到内容分页队列中
	 * @param task 当前任务对象
	 * @param orginHtml 原始HTML内容
	 */
	private void populatePaginationList(Task task,String orginHtml) {
		List<String> urlList = task.getController().getParseHtmlTool().getPaginationList(orginHtml);
		if(!CollectionUtils.isEmpty(urlList)){
			for(String url : urlList){
				getUrlQueue().addUnVisitedUrl(repairUrl(task,url));
			}
		}
	}
	/**
	 * 补全URL
	 * @param task 任务
	 * @param pUrl 原始URL
	 * @return 处理后URL
	 */
	private String repairUrl(Task task,String pUrl){
		if(!pUrl.contains("http://")){
			//如果有分页内容补全URL
			if(StringUtils.hasText(task.getController().getCrawlScope().getPaginationRepairUrl())){
				pUrl = task.getController().getCrawlScope().getPaginationRepairUrl()+pUrl;
			}else{
				//若pUrl是相对于根路径路径 
				if(pUrl.contains("/") && pUrl.indexOf("/") == 0){
					pUrl = task.getController().getCrawlScope().getRepairPageUrl() + pUrl;
				}else{
					//没有则，取得当前URL
					String tempPageUrl = "";
					if(task.getUrl().contains("http://")){
						tempPageUrl = task.getUrl().trim();
					}else{
						tempPageUrl = task.getController().getCrawlScope().getRepairPageUrl()+task.getUrl().trim();
					}
					pUrl = tempPageUrl.substring(0, tempPageUrl.lastIndexOf("/")+1)+pUrl;
				}
			}
		}
		return pUrl;
	}
	/**
	 * 处理任务
	 * @param task
	 */
	private void processorHTML(Task task){
		log.info("=========处理任务开始=========");
		try{
			String tempOrginHtml = "";
			String tempPageUrl = "";
			while(!getUrlQueue().isEmpty() && !Thread.interrupted()){
				tempPageUrl = getUrlQueue().unVisitedUrlDeQueue();
				tempOrginHtml = fetchHTML(task,tempPageUrl);
				if(StringUtils.hasText(tempOrginHtml)){
					//提取内容分页中的链接,并加入到队列中
					populatePaginationList(task,tempOrginHtml);
					savePaginationContent(task,tempOrginHtml);
					getUrlQueue().addVisitedUrl(tempPageUrl);
				}
			}
		}finally{
			log.info("=========处理任务结束=========");
			getUrlQueue().clear();
			pageUrlQueueThreadLocal.set(null);
		}
	}
	/**
	 * 保存内容分页
	 * @param task 任务
	 * @param orginHtml 原始HTML
	 */
	private void savePaginationContent(Task task,String orginHtml){
		//取得指定区域内容
		String html = task.getController().getParseHtmlTool().getTagetHtml(orginHtml);
		if(StringUtils.hasText(html)){
			html = extractorContentResource(task, html);
			html = filterContentLink(task, html);
			//过滤指定区域内容
			html = task.getController().getParseHtmlTool().filterTagetHtml(html);
			task.getContentBean().getContentList().add(html);
		}
	}
	/**
	 * 过滤内容中的连接
	 * @param task 任务
	 * @param html 原始内容
	 * @return 过滤后的内容
	 */
	private String filterContentLink(Task task, String html) {
		//替换掉内容中的超链接
		if(task.getController().getCrawlScope().isReplaceHtmlLink()){
			html = task.getController().getParseHtmlTool().replaceHtmlLink(html);
		}
		return html;
	}

	/**
	 * 提取内容中的资源
	 * @param task 任务
	 * @param html 原始内容
	 * @return 提取后的内容
	 */
	private String extractorContentResource(Task task, String html) {
		//替换原图片地址为本地图片地址，并将原图片地址保存在任务中，以备下一步使用
		if(task.getController().getCrawlScope().isExtractContentRes()){
			String savePath = task.getController().getCrawlScope().getSavePath();
			task.getController().getParseHtmlTool().replaceHtmlResource(html,savePath,task.getContentBean().getResMap());
			html = (String)task.getContentBean().getResMap().get(Constants.RES_HTML_KEY);
		}else{
			html = task.getController().getParseHtmlTool().getHtmlResource(html,task.getContentBean().getResMap(),task.getController().getCrawlScope().getRepairImageUrl());
		}
		return html;
	}
	
	/**
	 * 根据任务URL地址,取得HTML
	 * @param task 任务
	 */
	private String fetchHTML(Task task,String pUrl){
		log.info("=========下载分页内容,提取原始html=========");
		HttpGet httpGet = null;
		String html = "";
		try {
			httpGet = HttpGetHandler.getHttpGet(new URI(pUrl.trim()));
			html = task.getController().getHttpClient().execute(httpGet, task.getController().getHandler());
		} catch (Exception e) {
			e.printStackTrace();
		}finally{
			if(null != httpGet){
				httpGet.abort();
			}
		}
		return html;
	}
	
	/**
	 * 取得当前主线程的SimpleUrlQueue对象
	 * @return 当前主线程的SimpleUrlQueue对象
	 */
	private static SimpleUrlQueue getUrlQueue(){
		if(pageUrlQueueThreadLocal.get() == null){
			SimpleUrlQueue pageUrlQueue = new SimpleUrlQueue();
			pageUrlQueueThreadLocal.set(pageUrlQueue);
			return pageUrlQueue;
		}else{
		return pageUrlQueueThreadLocal.get();
		}
	} 
	public static void main(String[] argsStrings ) {
		 String testString = "/test/test.html";
		 String testString2 = "test.html";
		 String testString3 = "?223";
		 String testString4 = ".223";
		 System.out.println(testString.contains("\\/"));
		 System.out.println(testString4.contains("\\."));
		 System.out.println(testString.indexOf("/"));
	}
}
