package org.javacoo.crawler.util.parser;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.URISyntaxException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.CopyOnWriteArrayList;

import net.sf.json.JSONArray;

import org.apache.commons.io.FilenameUtils;
import org.apache.commons.lang.StringUtils;
import org.htmlparser.Node;
import org.htmlparser.NodeFilter;
import org.htmlparser.Parser;
import org.htmlparser.PrototypicalNodeFactory;
import org.htmlparser.filters.HasAttributeFilter;
import org.htmlparser.filters.NodeClassFilter;
import org.htmlparser.filters.TagNameFilter;
import org.htmlparser.nodes.RemarkNode;
import org.htmlparser.tags.BodyTag;
import org.htmlparser.tags.CompositeTag;
import org.htmlparser.tags.ImageTag;
import org.htmlparser.tags.LinkTag;
import org.htmlparser.util.NodeList;
import org.htmlparser.util.ParserException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.CollectionUtils;

import org.javacoo.crawler.constants.Constants;
import org.javacoo.crawler.data.ParamBean;
import org.javacoo.crawler.filter.FieldFilter;
import org.javacoo.crawler.filter.Filter;
import org.javacoo.crawler.filter.LinkAreaFilter;
import org.javacoo.crawler.util.generator.DefaultGenerator;
import org.javacoo.crawler.util.generator.Generator;
import org.javacoo.crawler.util.parser.tag.EmbedTag;
/**
 * HTML解析工具,HtmlParser实现类
 * @author javacoo
 * @since 2011-10-31
 */
public class HtmlParserImpl implements ParseHtmlTool{
	private static final Logger log = LoggerFactory.getLogger(HtmlParserImpl.class);
	/**单标签标志*/
	private final static String SINGLE_TAG = "singleTag";
	/**采集参数bean*/
	private ParamBean paramBean;
	/**唯一标示生成接口,默认实现*/
	private Generator generator = new DefaultGenerator();
	private PrototypicalNodeFactory factory;
	/**
	 * HTML解析类构造方法
	 * @param filterList 过滤器集合
	 */
	public HtmlParserImpl(List<Filter> filterList){
		factory = new PrototypicalNodeFactory();  
		factory.registerTag(new EmbedTag());  
		parseRequestParam(filterList);
	}
	/**
	 * HTML解析类构造方法
	 * @param generator 唯一标示生成器
	 * @param filterList 过滤器集合
	 */
	public HtmlParserImpl(Generator generator,List<Filter> filterList){
		this.generator = generator;
		parseRequestParam(filterList);
	}
	
	/**
	 * 取得标题集合
	 * @param orginHtml 原始HTML
	 * @return 标题集合
	 */
	public List<String> getTitleList(String orginHtml) {
		orginHtml = getHtmlByFilter(paramBean.getLinksetStartMap(), paramBean.getLinksetEndMap(),orginHtml);
		if (StringUtils.isNotBlank(orginHtml)) {
			return getUrlOrTitleListByType(orginHtml,Constants.TITLE_KEY);
		}
		return null;
	}
	
	/**
	 * 取得连接集合
	 * @param orginHtml 原始HTML
	 * @return 连接集合
	 */
	public List<String> getUrlList(String orginHtml) {
		orginHtml = getHtmlByFilter(paramBean.getLinksetStartMap(), paramBean.getLinksetEndMap(),orginHtml);
		if (StringUtils.isNotBlank(orginHtml)) {
			return getUrlOrTitleListByType(orginHtml,Constants.LINK_KEY);
		}
		return null;
	}
	/**
	 * 取得原始HTML中指定条件的值并放在对应字段
	 * @param orginHtml 原始HTML
	 * @return 字段 值对照MAP
	 */
	public Map<String,List<String>> getFieldValues(String orginHtml){
		log.info("========取得原始HTML中指定条件的值并放在对应字段=========");
		Map<String,List<String>> resultMap = new HashMap<String, List<String>>();
		try {
			Parser parser = new Parser();
			String field = null;
			String tempHtml = null;
			for(Iterator<String> it = paramBean.getFieldStartMap().keySet().iterator(); it.hasNext();){
				field = it.next();
				// 第一步取得指定属性/标签内容
				tempHtml = filterRequire(paramBean.getFieldStartMapByField(field),parser,orginHtml);
				// 第二步过滤指定属性/标签内容
				tempHtml = filterThrowAway(paramBean.getFieldEndMapByField(field), parser, tempHtml);
				resultMap.put(field, getFieldValueList(parser,paramBean.getFieldStartMapByField(field),tempHtml));
			}
		} catch (ParserException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		return resultMap;
	}
	/**
     * 取得指定区域的HTML内容
     * @param orginHtml 原始HTML
     * @return 指定区域的HTML内容
     * @throws ParserException
     */
	public String getHtml(String orginHtml) {
		if (StringUtils.isNotBlank(orginHtml)) {
			orginHtml = getHtmlByFilter(paramBean.getContentStartMap(), paramBean.getContentEndMap(),orginHtml);
		}
		return orginHtml;
	}
	/**
     * 取得指定区域的HTML内容
     * @return 指定区域的HTML内容
     */
	public String getTagetHtml(String orginHtml){
		log.info("========取得指定区域的HTML内容=========");
		if (StringUtils.isNotBlank(orginHtml)) {
			try {
				Parser parser = new Parser();
				orginHtml = filterRequire(paramBean.getContentStartMap(), parser,orginHtml);
				log.info(orginHtml);
			} catch (ParserException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		return orginHtml;
	}
	/**
     * 过滤指定区域的HTML内容
     * @return 过滤的HTML内容
     */
	public String filterTagetHtml(String orginHtml){
		log.info("========过滤指定区域的HTML内容=========");
		if (StringUtils.isNotBlank(orginHtml)) {
			try {
				Parser parser = new Parser();
				orginHtml = filterThrowAway(paramBean.getContentEndMap(),parser, orginHtml);
				orginHtml = filterOtherTag(parser, orginHtml);
				log.info(orginHtml);
			} catch (ParserException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		return orginHtml;
		
	}
	/**
	 * 取得无格式的HTML内容
	 * @param orginHtml 原始HTML内容
	 * @return 无格式的HTML内容
	 */
	public String getPlainText(String orginHtml){
		log.info("========取得无格式的HTML内容=========");
		String resultStr = "";
		try {
			Parser parser = new Parser();
			parser.setInputHTML(orginHtml);
			StringBuilder sb = new StringBuilder();
			while(parser.elements().hasMoreNodes()){
				sb.append(StringUtils.trim(parser.elements().nextNode().toPlainTextString()));
			}
			resultStr = sb.toString();
		} catch (ParserException e) {
			e.printStackTrace();
		}
		return resultStr.replaceAll("\\s", "");
	}
	/**
	 * 取得HTML内容中的资源
	 * @param orginHtml 原始HTML内容
	 * @param resMap 资源MAP
	 * @return MAP
	 */
	public String getHtmlResource(String orginHtml,Map<String,Object> resMap,String repairUrl){
		log.info("========取得HTML内容中的资源=========");
		if (StringUtils.isNotBlank(orginHtml)) {
			try {
				Parser parser = new Parser();
				parser.setInputHTML(orginHtml);
				orginHtml = extractImageTag(parser,orginHtml,resMap,repairUrl);
				extractLinkRes(parser, resMap);
				orginHtml = extractObject(parser,orginHtml, resMap);
			} catch (ParserException e) {
				e.printStackTrace();
			}
		}
		return orginHtml;
		
	}
	/**
     * 取得指定HTML内容中的摘要信息
     * @param orginHtml 原始HTML信息
     * @return 指定区域的HTML内容
     */
	public String getBrief(String orginHtml) {
		log.info("========取得指定HTML内容中的摘要信息=========");
		String returnHtml = "";
		if (StringUtils.isNotBlank(orginHtml)) {
			if(CollectionUtils.isEmpty(paramBean.getBriefStartMap())){//如果摘要采集参数为空,则提取内容中指定字数字符作为摘要信息
				returnHtml = getTagetHtml(orginHtml);
				returnHtml = filterTagetHtml(returnHtml);
				returnHtml = getPlainText(returnHtml);
			}else{
				returnHtml = getHtmlByFilter(paramBean.getBriefStartMap(), paramBean.getBriefEndMap(),orginHtml);
				returnHtml = getPlainText(returnHtml);
			}
			returnHtml = StringUtils.left(returnHtml, Constants.BRIEF_NUM);
		}
		return returnHtml;
	}
	/**
     * 取得指定HTML内容中的分页链接列表信息
     * @param orginHtml 原始HTML信息
     * @return 分页链接列表
     */
	public List<String> getPaginationList(String orginHtml) {
		log.info("========取得指定HTML内容中的分页链接列表信息=========");
		if (StringUtils.isNotBlank(orginHtml)) {
			if(!CollectionUtils.isEmpty(paramBean.getPaginationStartMap())){
				String pageAreaHtml = getHtmlByFilter(paramBean.getPaginationStartMap(), paramBean.getPaginationEndMap(),orginHtml);
				return getUrlOrTitleListByType(pageAreaHtml,Constants.LINK_KEY);
			}
		}
		return null;
	}
	
	/**
	 * 提取内容中图片标签内容
	 * @param parser Parser对象
	 * @param orginHtml 原始HTML
	 * @param resMap 资源MAP
	 * @param repairUrl 资源补全地址
	 * @throws ParserException 
	 */
	private String extractImageTag(Parser parser,String orginHtml, Map<String, Object> resMap,String repairUrl) throws ParserException {
		log.info("========提取内容中图片标签内容=========");
		Map<String,String> resultMap = new ConcurrentHashMap<String,String>();
		NodeFilter filter = new NodeClassFilter(ImageTag.class);
		NodeList nodes = parser.extractAllNodesThatMatch(filter);
		ImageTag imageTag = null;
		String orginImgUrl = null;
		String orginImgTagHtml = null;
		String newImgTagHtml = null;
		for (int i = 0; i < nodes.size(); i++) {
			imageTag = (ImageTag)nodes.elementAt(i);
			orginImgUrl = imageTag.getImageURL();
			if(StringUtils.isNotBlank(orginImgUrl)){
				if(!orginImgUrl.contains("http://")){
					orginImgTagHtml = imageTag.toHtml();
					resultMap.put(repairUrl+orginImgUrl, "");
					//设置新URL
					imageTag.setImageURL(repairUrl+orginImgUrl);
					newImgTagHtml =  imageTag.toHtml();
					orginHtml = StringUtils.replace(orginHtml, orginImgTagHtml, newImgTagHtml);
				}else{
					resultMap.put(orginImgUrl, "");
				}
			}
			
		}
		if(null != resMap.get(Constants.RES_IMAGES_MAP_KEY)){
			Map<String,String> addResMap = (Map<String,String>)resMap.get(Constants.RES_IMAGES_MAP_KEY);
			addResMap.putAll(resultMap);
		}else{
			resMap.put(Constants.RES_IMAGES_MAP_KEY, resultMap);
		}
		parser.reset();
		return orginHtml;
	}
	/**
	 * 提取内容连接中的资源
	 * @param parser Parser对象
	 * @param resMap 资源MAP
	 * @throws ParserException 
	 */
	private void extractLinkRes(Parser parser, Map<String, Object> resMap) throws ParserException {
		log.info("========提取内容连接中的资源=========");
		Map<String,String> resultMap = new ConcurrentHashMap<String,String>();
		NodeFilter filter = new NodeClassFilter(LinkTag.class){
			public boolean accept(Node node){
				if(node instanceof LinkTag){
					if(StringUtils.isNotBlank(((LinkTag) node).getLink()) && Constants.EXTRACT_RES_TYPE.contains(FilenameUtils.getExtension(((LinkTag) node).getLink()).toLowerCase())){
						return true;
					}else{
						return false;
					}
				}else{
					return false;
				}
			}
		};
		NodeList nodes = parser.extractAllNodesThatMatch(filter);
		LinkTag linkTag = null;
		for (int i = 0; i < nodes.size(); i++) {
			linkTag = (LinkTag)nodes.elementAt(i);
			resultMap.put(linkTag.getLink(), "");
		}
		resMap.put(Constants.RES_ATTAC_MAP_KEY, resultMap);
		parser.reset();
	}
	/**
	 * 提取内容中对象资源，如，媒体资源
	 * @param parser Parser对象
	 * @param resMap 资源MAP
	 * @throws ParserException 
	 */
	private String extractObject(Parser parser,String orginHtml, Map<String, Object> resMap) throws ParserException {
		log.info("========提取内容中对象资源=========");
		parser.setNodeFactory(factory);
		Map<String,String> resultMap = new ConcurrentHashMap<String,String>();
		NodeFilter filter = new NodeClassFilter(EmbedTag.class){
			public boolean accept(Node node){
				if(node instanceof EmbedTag){
					if(StringUtils.isNotBlank(((EmbedTag) node).getSrc()) && Constants.EXTRACT_MEDIA_RES_TYPE.contains(FilenameUtils.getExtension(((EmbedTag) node).getSrc()).toLowerCase())){
						return true;
					}else{
						return false;
					}
				}else{
					return false;
				}
			}
		};
		NodeList nodes = parser.extractAllNodesThatMatch(filter);
		EmbedTag embedTag = null;
		for (int i = 0; i < nodes.size(); i++) {
			embedTag = (EmbedTag)nodes.elementAt(i);
			resultMap.put(embedTag.getSrc(), "");
			//去掉原始OBJ对象
			orginHtml = StringUtils.replace(orginHtml, embedTag.toHtml(), "");
		}
		resMap.put(Constants.RES_MEDIA_MAP_KEY, resultMap);
		parser.reset();
		return orginHtml;
	}
	/**
	 * 替换HTML内容中资源链接地址为指定地址，并返回包含了替换后HTML，原始资源链接与替换后资源链接对照的MAP对象
	 * @param orginHtml 原始HTML内容
	 * @param savePath 资源保存路径
	 * @param resMap 资源MAP
	 * @return MAP
	 */
	public void replaceHtmlResource(String orginHtml,String savePath,Map<String,Object> resMap){
		if(StringUtils.isNotBlank(orginHtml)){
			try {
				Parser parser = new Parser();
				parser.setInputHTML(orginHtml);
				orginHtml = replaceImages(orginHtml, savePath, resMap, parser);
				orginHtml = replaceLinkRes(orginHtml, savePath, resMap, parser);
				orginHtml = replaceObject(orginHtml, savePath, resMap, parser);
			} catch (ParserException e) {
				e.printStackTrace();
			}
			resMap.put(Constants.RES_HTML_KEY, orginHtml);
		}
	}
	/**
	 * 替换图片
	 * @param orginHtml 原始HTML
	 * @param savePath 文件保存路径
	 * @param resMap 资源MAP
	 * @param parser Parser对象
	 * @return
	 * @throws ParserException
	 */
	private String replaceImages(String orginHtml, String savePath,
			Map<String, Object> resMap, Parser parser) throws ParserException {
		log.info("========替换图片=========");
		//图片地址MAP：KEY为新图片地址，value为源图片地址
		Map<String,String> resultMap = new ConcurrentHashMap<String,String>();
		NodeFilter filter = new NodeClassFilter(ImageTag.class);
		NodeList nodes = parser.extractAllNodesThatMatch(filter);
		//原始图片标签HTML
		String orginImgTagHtml = null;
		//原始图片标签URL
		String orginImgTagUrl = null;
		//新图片标签HTML
		String newImgTagHtml = null;
		//新图片标签URL
		String newImgTagUrl = null;
		ImageTag imageTag = null;
		for (int i = 0; i < nodes.size(); i++) {
			imageTag = (ImageTag)nodes.elementAt(i);
			orginImgTagHtml = imageTag.toHtml();
			orginImgTagUrl = imageTag.getImageURL();
			newImgTagUrl = populateSavePath(orginImgTagUrl,savePath);
			//设置新URL
			imageTag.setImageURL(newImgTagUrl);
			newImgTagHtml =  imageTag.toHtml();
			resultMap.put(newImgTagUrl,orginImgTagUrl);
			orginHtml = StringUtils.replace(orginHtml, orginImgTagHtml, newImgTagHtml);
		}
		if(null != resMap.get(Constants.RES_IMAGES_MAP_KEY)){
			Map<String,String> addResMap = (Map<String,String>)resMap.get(Constants.RES_IMAGES_MAP_KEY);
			addResMap.putAll(resultMap);
		}else{
			resMap.put(Constants.RES_IMAGES_MAP_KEY, resultMap);
		}
		parser.reset();
		return orginHtml;
	}
	/**
	 * 替换内容中连接资源
	 * @param orginHtml 原始HTML
	 * @param savePath 文件保存路径
	 * @param resMap 资源MAP
	 * @param parser Parser对象
	 * @return
	 * @throws ParserException
	 */
	private String replaceLinkRes(String orginHtml, String savePath,
			Map<String, Object> resMap, Parser parser) throws ParserException {
		log.info("========替换内容中连接资源=========");
		//资源地址MAP：KEY为新资源地址，value为源资源地址
		Map<String,String> resultMap = new ConcurrentHashMap<String,String>();
		NodeFilter filter = new NodeClassFilter(LinkTag.class){
			public boolean accept(Node node){
				if(node instanceof LinkTag){
					if(StringUtils.isNotBlank(((LinkTag) node).getLink()) && Constants.EXTRACT_RES_TYPE.contains(FilenameUtils.getExtension(((LinkTag) node).getLink()).toLowerCase())){
						return true;
					}else{
						return false;
					}
				}else{
					return false;
				}
			}
		};
		NodeList nodes = parser.extractAllNodesThatMatch(filter);
		//原始资源标签HTML
		String orginTagHtml = null;
		//原始资源标签URL
		String orginTagUrl = null;
		//新资源标签HTML
		String newTagHtml = null;
		//新资源标签URL
		String newTagUrl = null;
		LinkTag linkTag = null;
		for (int i = 0; i < nodes.size(); i++) {
			linkTag = (LinkTag)nodes.elementAt(i);
			orginTagHtml = linkTag.toHtml();
			orginTagUrl = linkTag.getLink();
			newTagUrl = populateSavePath(orginTagUrl,savePath);
			//设置新URL
			linkTag.setLink(newTagUrl);
			newTagHtml =  linkTag.toHtml();
			resultMap.put(newTagUrl,orginTagUrl);
			orginHtml = StringUtils.replace(orginHtml, orginTagHtml, newTagHtml);
		}
		resMap.put(Constants.RES_ATTAC_MAP_KEY, resultMap);
		parser.reset();
		return orginHtml;
	}
	/**
	 * 替换内容中对象资源
	 * @param orginHtml 原始HTML
	 * @param savePath 文件保存路径
	 * @param resMap 资源MAP
	 * @param parser Parser对象
	 * @return
	 * @throws ParserException
	 */
	private String replaceObject(String orginHtml, String savePath,
			Map<String, Object> resMap, Parser parser) throws ParserException {
		log.info("========替换内容中对象资源=========");
		parser.setNodeFactory(factory);
		//资源地址MAP：KEY为新资源地址，value为源资源地址
		Map<String,String> resultMap = new ConcurrentHashMap<String,String>();
		NodeFilter filter = new NodeClassFilter(EmbedTag.class){
			public boolean accept(Node node){
				if(node instanceof EmbedTag){
					if(StringUtils.isNotBlank(((EmbedTag) node).getSrc()) && Constants.EXTRACT_MEDIA_RES_TYPE.contains(FilenameUtils.getExtension(((EmbedTag) node).getSrc()).toLowerCase())){
						return true;
					}else{
						return false;
					}
				}else{
					return false;
				}
			}
		};
		NodeList nodes = parser.extractAllNodesThatMatch(filter);
		
		//原始资源标签HTML
		String orginTagHtml = null;
		//原始资源标签URL
		String orginTagUrl = null;
		//新资源标签HTML
		String newTagHtml = null;
		//新资源标签URL
		String newTagUrl = null;
		EmbedTag embedTag = null;
		for (int i = 0; i < nodes.size(); i++) {
			embedTag = (EmbedTag)nodes.elementAt(i);
			orginTagHtml = embedTag.toHtml();
			orginTagUrl = embedTag.getSrc();
			//newTagUrl = populateSavePath(orginTagUrl,savePath);
			newTagUrl = orginTagUrl;//暂时不替换视频对象
			//设置新URL
			embedTag.setSrc(newTagUrl);
			newTagHtml =  embedTag.toHtml();
			resultMap.put(newTagUrl,orginTagUrl);
			//去掉原来OBJ对象
			orginHtml = StringUtils.replace(orginHtml, orginTagHtml, "");
			//orginHtml = StringUtils.replace(orginHtml, orginTagHtml, newTagHtml);
		}
		resMap.put(Constants.RES_MEDIA_MAP_KEY, resultMap);
		parser.reset();
		return orginHtml;
	}
	/**
	 * 组装新图片地址
	 * @param orginImgTagUrl 原始图片地址
	 * @param savePath 保存路径
	 * @return
	 */
	private String populateSavePath(String orginImgTagUrl,String savePath){
		StringBuilder resultStr = new StringBuilder();
		//是否重命名
		boolean replaceName = false;
		if(StringUtils.isNotBlank(orginImgTagUrl) && StringUtils.isNotBlank(savePath) && orginImgTagUrl.contains(Constants.IMAGE_SPLIT)){
			if(replaceName){
				resultStr.append(savePath).append(generator.next()).append(Constants.IMAGE_SPLIT).append(FilenameUtils.getExtension(orginImgTagUrl));
			}else{
				resultStr.append(savePath).append(FilenameUtils.getName(orginImgTagUrl));
			}
			
		}
		return resultStr.toString();
	}
	/**
	 * 替换内容中的超链接
	 * @param orginHtml 原始HTML内容
	 * @return 去掉超链接后的HTML
	 */
	public String replaceHtmlLink(String orginHtml){
		log.info("========替换内容中的超链接=========");
		if(StringUtils.isNotBlank(orginHtml)){
			try {
				Parser parser = new Parser();
				parser.setInputHTML(orginHtml);
				NodeFilter filter = new NodeClassFilter(LinkTag.class);
				NodeList nodes = parser.extractAllNodesThatMatch(filter);
				for (int i = 0; i < nodes.size(); i++) {
					LinkTag linkTag = (LinkTag)nodes.elementAt(i);
					orginHtml = StringUtils.replace(orginHtml, linkTag.toHtml(), linkTag.getLinkText());
				}
			} catch (ParserException e) {
				e.printStackTrace();
			}
		}
		return orginHtml;
	}
	/**
	 * 取得连接标题Map，如果有标题图片则放到资源MAP中
	 * @param orginHtml 原始HTML
	 * @param savePath 标题图片保存路径
	 * @return 连接标题Map集合
	 */
	public List<Map<String,String>> getUrlAndTitleMap(String orginHtml,String savePath){
		return getUrlAndTitleAndImagesMap(orginHtml,savePath);
	}
	/**
	 * 解析采集参数,并封装到ParamBean
	 * @param acqu 原始采集参数
	 * @return 采集参数封装bean
	 */
	private void parseRequestParam(List<Filter> filterList){
		log.info("========解析采集参数,并封装到ParamBean=========");
		paramBean = new ParamBean();
		String fetchAreaString = null;
		String filterAreaString = null;
		for(Filter filter : filterList){
			fetchAreaString = filter.getFilter().get(Constants.FETCH_AREA_KEY);
			filterAreaString = filter.getFilter().get(Constants.FILTER_AREA_KEY);
			if(Constants.FILTER_NAME_LINK_AREA.equals(filter.getFilterName())){
				if(StringUtils.isNotBlank(fetchAreaString)){
					paramBean.setLinksetStartMap(populateParamMap(fetchAreaString.trim()));
				}
				if(StringUtils.isNotBlank(filterAreaString)){
					paramBean.setLinksetEndMap(populateParamMap(filterAreaString.trim()));
				}
			}
			if(Constants.FILTER_NAME_CONTENT_AREA.equals(filter.getFilterName())){
				if(StringUtils.isNotBlank(fetchAreaString)){
					paramBean.setContentStartMap(populateParamMap(fetchAreaString.trim()));
				}
				if(StringUtils.isNotBlank(filterAreaString)){
					paramBean.setContentEndMap(populateParamMap(filterAreaString.trim()));
				}
			}
			if(Constants.FILTER_NAME_BRIEF_AREA.equals(filter.getFilterName())){
				if(StringUtils.isNotBlank(fetchAreaString)){
					paramBean.setBriefStartMap(populateParamMap(fetchAreaString.trim()));
				}
				if(StringUtils.isNotBlank(filterAreaString)){
					paramBean.setBriefEndMap(populateParamMap(filterAreaString.trim()));
				}
			}
			if(Constants.FILTER_NAME_FIELD.equals(filter.getFilterName())){
				if(StringUtils.isNotBlank(fetchAreaString) && StringUtils.isNotBlank(filter.getFieldName())){
					paramBean.addFieldStartMap(filter.getFieldName(),populateParamMap(fetchAreaString.trim()));
				}
				if(!StringUtils.isEmpty(filterAreaString)){
					paramBean.addFieldEndMap(filter.getFieldName(),populateParamMap(filterAreaString.trim()));
				}
			}
			if(Constants.FILTER_NAME_PAGINATION_AREA.equals(filter.getFilterName())){
				if(StringUtils.isNotBlank(fetchAreaString)){
					paramBean.setPaginationStartMap(populateParamMap(fetchAreaString.trim()));
				}
				if(StringUtils.isNotBlank(filterAreaString)){
					paramBean.setPaginationEndMap(populateParamMap(filterAreaString.trim()));
				}
			}
		}
		
	}
	/**
	 * 得到连接标题MAP
	 * @param orginHtml 原始HTML
	 * @param savePath 标题图片保存路径
	 * @return 连接或者标题集合
	 */
	private List<Map<String,String>> getUrlAndTitleAndImagesMap(String html,String savePath) {
		log.info("========开始得到连接标题MAP=========savePath:"+savePath);
		log.info("========连接标题MAP=========getLinksetStartMap："+paramBean.getLinksetStartMap()+",getLinksetEndMap:"+paramBean.getLinksetEndMap());
		List<Map<String,String>> linkTitleMapList = new CopyOnWriteArrayList<Map<String,String>>();
		try {
			html = getHtmlByFilter(paramBean.getLinksetStartMap(), paramBean.getLinksetEndMap(),html);
			Parser parser = new Parser();
			parser.setInputHTML(html);
			
			Map<String,String> linkTitleMap = null;
			NodeFilter filter = new NodeClassFilter(LinkTag.class);
			NodeList nodes = parser.extractAllNodesThatMatch(filter);
			String newImagePath = "";
			String orignImagePath = "";
			String title = "";
			for (int i = 0; i < nodes.size(); i++) {
				LinkTag linkTag = (LinkTag)nodes.elementAt(i);

				orignImagePath = "";
				newImagePath = "";
				title = linkTag.getLinkText();
				
				//取得连接中的子标签
				NodeList childrenList = linkTag.getChildren();
				for(int j=0;j < childrenList.size();j++){
					Node node = childrenList.elementAt(j);
					//如果有图片
					if(null != node && node instanceof ImageTag){
						ImageTag imageTag = (ImageTag)node;
						orignImagePath = imageTag.getImageURL();
						newImagePath = populateSavePath(orignImagePath,savePath);
						title = linkTag.getAttribute("title");
						break;
					}
				}
				if(StringUtils.isNotBlank(linkTag.getLink()) && StringUtils.isNotBlank(title)){
					linkTitleMap = new ConcurrentHashMap<String, String>();
					linkTitleMap.put(Constants.NEW_IMAGE_KEY, newImagePath);
					linkTitleMap.put(Constants.ORIGN_IMAGE_KEY, orignImagePath);
					
					linkTitleMap.put(Constants.LINK_KEY, linkTag.getLink());
					linkTitleMap.put(Constants.TITLE_KEY, title);
					
					linkTitleMapList.add(linkTitleMap);
				}
			}
			return linkTitleMapList;
		} catch (ParserException e) {
			e.printStackTrace();
		}
		return null;
	}
	/**
	 * 得到地址集
	 * @param html html内容
	 * @param type 1 ：取得连接集合，2：取得标题集合
	 * @return 连接或者标题集合
	 */
	private List<String> getUrlOrTitleListByType(String html, String type) {
		List<String> resultList = new CopyOnWriteArrayList<String>();
		try {
			Parser parser = new Parser();
			parser.setInputHTML(html);
			NodeFilter filter = new NodeClassFilter(LinkTag.class);
			NodeList nodes = parser.extractAllNodesThatMatch(filter);
			for (int i = 0; i < nodes.size(); i++) {
				LinkTag linkTag = (LinkTag)nodes.elementAt(i);
				if(StringUtils.isNotBlank(linkTag.getLink())&& !"#".equals(linkTag.getLink().trim()) && !resultList.contains(linkTag.getLink().trim())){
					if(Constants.TITLE_KEY.equals(type) && StringUtils.isNotBlank(linkTag.getLinkText())){
						resultList.add(linkTag.getLinkText().trim());
					}else{
						resultList.add(linkTag.getLink().trim());
					}
				}
			}
			return resultList;
		} catch (ParserException e) {
			e.printStackTrace();
		}
		return null;
	}
	/**
     * 取得指定区域的HTML内容
     * @param tagMap 标签MAP
     * @param removeTagMap 要过滤的标签MAP
     * @param orginHtml 原始HTML
     * @return 指定区域的HTML内容
     * @throws ParserException
     */
	private String getHtmlByFilter(Map<String, String> tagMap,
			Map<String, String> removeTagMap, String orginHtml) {
		if(StringUtils.isNotBlank(orginHtml)){
			try {
				Parser parser = new Parser();
				// 第一步取得指定属性/标签内容
				orginHtml = filterRequire(tagMap,parser,orginHtml);
				// 第二步过滤指定属性/标签内容
				orginHtml = filterThrowAway(removeTagMap, parser, orginHtml);
				// 第三步过滤其他标签
				orginHtml = filterOtherTag(parser, orginHtml);
				log.info("=================================采集结果=======================================");
				log.info(orginHtml);
				return orginHtml;
			} catch (ParserException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}
		return "";
	}
	/**
	 * 采集指定属性/标签内容
	 * 如果没有找到，则在注释中查找
	 * @param tagMap 指定属性/标签
	 * @param orginHtml 原始HTML
	 * @param parser
	 * @return
	 * @throws ParserException
	 */
	private String filterRequire(Map<String, String> tagMap,
			Parser parser, String orginHtml) throws ParserException {
		String resultHtml = getNaturalHtml(tagMap,parser,orginHtml);
		//如果没有找到，则在注释内容中找
		if(StringUtils.isEmpty(resultHtml)){
			resultHtml = getRemarkHtml(parser,orginHtml);
			resultHtml = getNaturalHtml(tagMap,parser,resultHtml);
		}
		return resultHtml;
	}
	/**
	 * 在正常的HTML内容中采集指定属性/标签内容
	 * @param tagMap 指定属性/标签
	 * @param orginHtml 原始HTML
	 * @param parser
	 * @return
	 * @throws ParserException
	 */
	private String getNaturalHtml(Map<String, String> tagMap,
			Parser parser, String orginHtml)throws ParserException{
		if(null != tagMap && !tagMap.isEmpty()){
			log.info("========开始采集指定属性/标签内容=========");
			parser.setInputHTML(orginHtml);
			StringBuilder sb = new StringBuilder();
			// 第一步取得指定属性/标签内容
			List<NodeFilter> nodeFilterList = populateFilter(parser,tagMap);
			for(NodeFilter filter : nodeFilterList){
				appendHtmlByFilter(parser, filter, sb);
			}
			return sb.toString();
		}
		return orginHtml;
	}
	/**
	 * 取得注释内容
	 * @param parser 解析器
	 * @param orginHtml 原始HTML
	 * @return 注释内容
	 */
	private String getRemarkHtml(Parser parser,String orginHtml){
		log.info("========取得页面注释内容=========");
		NodeFilter filter = new NodeClassFilter(RemarkNode.class);
		StringBuilder sb = new StringBuilder();
		try {
			parser.setInputHTML(orginHtml);
	        NodeList nodes = parser.extractAllNodesThatMatch(filter);
	        for (int i = 0; i < nodes.size(); i++) {
				Node textnode = (Node) nodes.elementAt(i);
				sb.append(textnode.getText());
			}
	        return sb.toString();
		} catch (ParserException e) {
			e.printStackTrace();
		}
		return "";
	}
	/**
	 * 指定属性/标签内容
	 * @param tagMap 指定属性/标签
	 * @param orginHtml 原始HTML
	 * @param parser
	 * @return
	 * @throws ParserException
	 */
	private List<String> getFieldValueList(Parser parser,Map<String, String> tagMap,String orginHtml) throws ParserException {
		parser.setInputHTML(orginHtml);
		List<String> list = new CopyOnWriteArrayList<String>();
		// 取得指定属性/标签内容
		List<NodeFilter> nodeFilterList = populateFilter(parser,tagMap);
		for(NodeFilter filter : nodeFilterList){
			addPlainTextToList(parser, filter, list);
		}
		return list;
	}
	/**
	 * 过滤其他标签
	 * @param parser 解析器
	 * @param contentHtml 原始内容
	 * @return 过滤后内容
	 * @throws ParserException
	 */
	private String filterOtherTag(Parser parser, String contentHtml)
			throws ParserException {
		log.info("========开始过滤其他标签=========");
		//过滤注释
		NodeFilter filter = new NodeClassFilter(RemarkNode.class);
		contentHtml = removeHtmlByFilter(parser, filter, contentHtml);
		return contentHtml;
	}
	/**
	 * 过滤指定标签
	 * @param removeTagMap 指定标签属性MAP
	 * @param parser 解析器
	 * @param origin 
	 * @return
	 * @throws ParserException
	 */
	private String filterThrowAway(Map<String, String> removeTagMap,
			Parser parser, String orginHtml) throws ParserException {
		log.info("========开始过滤指定标签=========");
		if(null != removeTagMap && !removeTagMap.isEmpty()){
			List<NodeFilter> nodeFilterList;
			nodeFilterList = populateFilter(parser,removeTagMap);
			for(NodeFilter filter : nodeFilterList){
				orginHtml = removeHtmlByFilter(parser, filter, orginHtml);
			}
		}
		return orginHtml;
	}
	/**
	 * 根据参数MAP组装NodeFilter集合
	 * @param parser  Parser对象
	 * @param map  参数MAP
	 * @return NodeFilter集合
	 * @throws ParserException 
	 */
	private List<NodeFilter> populateFilter(Parser parser,Map<String, String> map) throws ParserException{
		List<NodeFilter> nodeFilterList = new CopyOnWriteArrayList<NodeFilter>();
		String tempKey = null;
		String tempValue = null;
		String[] tempValueArr = null;
		NodeFilter filter = null;
		for(Iterator<String> it = map.keySet().iterator(); it.hasNext();){
			tempKey = it.next();
			tempValue = map.get(tempKey);
			if(tempValue.contains("|")){
				tempValueArr = tempValue.split("\\|");
			}else{
				tempValueArr = new String[]{tempValue};
			}
			for(String value : tempValueArr){
				filter = populateFilter(tempKey,value);
				nodeFilterList.add(filter);
			}
		}
		return nodeFilterList;
	}
	
	/**
	 * 解析并组装采集参数，支持标签属性/值形式和标签名称形式，可混合使用
	 * <li>约定采集参数格式如下</li>
	 * <li>1，标签属性/值形式，如：class=articleList|tips,id=fxwb|fxMSN|fxMSN</li>
	 * <li>2，标签名称形式，如：div,p,span</li>
	 * <li>3，混合形式，如：class=articleList|tips,id=fxwb|fxMSN|fxMSN,div,p,span</li>
	 * @param paramStr 参数字符串
	 */
	private Map<String, String> populateParamMap(String paramStr) {
		Map<String, String> paramMap = new ConcurrentHashMap<String, String>();
		String[] paramStrArr = paramStr.split(",");
		String[] tempStrArr = null;
		StringBuilder sb = new StringBuilder();
		for(String temp : paramStrArr){
			if(temp.contains("=")){
				tempStrArr = temp.split("=");
				paramMap.put(tempStrArr[0], tempStrArr[1]);
			}else{
				if(StringUtils.isNotBlank(temp)){
					sb.append(temp).append("|");
				}
			}
		}
		if(StringUtils.isNotBlank(sb.toString())){
			paramMap.put(SINGLE_TAG, sb.substring(0, sb.length() - 1));
		}
		return paramMap;
	}
	/**
	 * 组装过滤器
	 * @param key 键
	 * @param value 值,支持正则表达式
	 * @return 过滤器
	 */
	private NodeFilter populateFilter(final String key,final String value) {
		NodeFilter filter;
		if(SINGLE_TAG.equals(key)){
			filter =  new TagNameFilter(value);
		}else{
			filter = new HasAttributeFilter(){
				public boolean accept(Node node) {
					if(node instanceof CompositeTag){
						CompositeTag tag = (CompositeTag)node;
						//如果匹配 则返回TRUE
						if(StringUtils.isNotBlank(tag.getAttribute(key)) && tag.getAttribute(key).matches(value)){
							return true;
						}else{
							return false;
						}
					}else{
						return false;
					}
				}
			};
		}
		return filter;
	}
	/**
     * 过滤指定属性标签HTML
     * @param parser 解析器
     * @param filter 属性过滤器
     * @param orginHtml 原始HTML
     * @return 过滤后HTML
     * @throws ParserException
     */
	private String removeHtmlByFilter(Parser parser, NodeFilter filter,String orginHtml) throws ParserException {
		parser.setInputHTML(orginHtml);
		NodeList nodes = parser.extractAllNodesThatMatch(filter);
		for (int i = 0; i < nodes.size(); i++) {
			Node textnode = (Node) nodes.elementAt(i);
			orginHtml = StringUtils.remove(orginHtml, textnode.toHtml());
		}
		return orginHtml;
	}
	/**
	 * 查找指定属性，指定属性值正则表达式匹配的所有CompositeTa
	 * @param node 节点
	 * @param tagList CompositeTa集合
	 * @param attributeName 属性
	 * @param attributeValueMatches 属性值正则表达式
	 */
	private void findAllCompositeTagByMatches(Node node,List<CompositeTag> tagList,String attributeName,String attributeValueMatches){
		if(node instanceof CompositeTag){
			CompositeTag tag = (CompositeTag)node;
			//查找指定属性，指定属性值正则表达式的节点
			if(StringUtils.isNotBlank(tag.getAttribute(attributeName)) && tag.getAttribute(attributeName).matches(attributeValueMatches)){
				tagList.add(tag);
			}
		}
		NodeList nodeList = node.getChildren();
		for(int i=0;i<nodeList.size();i++){
			Node nextNode = (Node) nodeList.elementAt(i);
			if(null != nextNode.getChildren()){
				findAllCompositeTagByMatches(nextNode,tagList,attributeName,attributeValueMatches);
			}
		}
	}
	/**
	 * 取得所有指定属性/标签的HTML
	 * @param parser 解析器
	 * @param filter 过滤器
	 * @param sb 
	 * @throws ParserException
	 */
	private void appendHtmlByFilter(Parser parser, NodeFilter filter,
			StringBuilder sb) throws ParserException {
		log.info("========开始取得所有指定属性/标签的HTML=========");
		NodeList nodes = parser.extractAllNodesThatMatch(filter);
		for (int i = 0; i < nodes.size(); i++) {
			Node textnode = (Node) nodes.elementAt(i);
			sb.append(textnode.toHtml());
			log.info("=========================HTML：========="+textnode.toHtml());
		}
	}
	/**
	 * 取得所有指定属性/标签的HTML
	 * @param parser 解析器
	 * @param filter 过滤器
	 * @param sb 
	 * @throws ParserException
	 */
	private void addPlainTextToList(Parser parser, NodeFilter filter,List<String> list) throws ParserException {
		NodeList nodes = parser.extractAllNodesThatMatch(filter);
		for (int i = 0; i < nodes.size(); i++) {
			Node textnode = (Node) nodes.elementAt(i);
			if(StringUtils.isNotBlank(textnode.toPlainTextString())){
				list.add(textnode.toPlainTextString().replaceAll("\\s", ""));
			}
		}
	}

	/**
	 * 解析并组装采集参数，支持标签属性/值形式和标签名称形式，可混合使用
	 * <li>约定采集参数格式如下</li>
	 * <li>1，标签属性/值形式，如：class=articleList.*|tips,id=fxwb|fxMSN|fxMSN
	 *      注意：属性值 支持正则表达式</li>
	 * <li>2，标签名称形式，如：div,p,span</li>
	 * <li>3，混合形式，如：class=articleList|tips,id=fxwb|fxMSN|fxMSN,div,p,span</li>
	 * @param paramMap 参数map
	 * @param str 参数字符串
	 */
	private void populateParamMap(Map<String, String> paramMap,String paramStr) {
		String[] paramStrArr = paramStr.split(",");
		String[] tempStrArr = null;
		StringBuilder sb = new StringBuilder();
		for(String temp : paramStrArr){
			if(temp.contains("=")){
				tempStrArr = temp.split("=");
				paramMap.put(tempStrArr[0], tempStrArr[1]);
			}else{
				if(StringUtils.isNotBlank(temp)){
					sb.append(temp).append("|");
				}
			}
		}
		if(StringUtils.isNotBlank(sb.toString())){
			paramMap.put(SINGLE_TAG, sb.substring(0, sb.length() - 1));
		}
	}
		
    /**
     * 测试方法-打开文件并返回内容
     * @param szFileName 文件绝对地址
     * @param charset 字符集
     * @return 内容
     */
	public static String openFile(String szFileName,String charset) {
		try {
			BufferedReader bis = new BufferedReader(new InputStreamReader(
					new FileInputStream(new File(szFileName)), charset));
			StringBuilder szContent = new StringBuilder();
			String szTemp;

			while ((szTemp = bis.readLine()) != null) {
				szContent.append(szTemp).append("\n");
			}
			bis.close();
			return szContent.toString();
		} catch (Exception e) {
			return "";
		}
	}
	/**
	 * 测试取得连接地址和标题
	 * @throws ParserException
	 */
	public void testFetchLinkAndTitle() throws ParserException{
//		String html = openFile("F:\\temp\\菜系 - 中华菜系.htm","UTF-8");
//		String result = "";
//		Map<String, String> map = new HashMap<String, String>();
//		map.put("class", "lp_result_list");
//		Map<String, String> notMap = new HashMap<String, String>();
//		notMap.put("singleTag", "h3|strong|span");
		String html = openFile("F:\\temp\\美食视频__中华美食网_视频.htm","GB2312");
		String result = "";
		Map<String, String> map = new HashMap<String, String>();
		map.put("class", "vl-mlist");
		Map<String, String> notMap = new HashMap<String, String>();
		//notMap.put("singleTag", "h3|strong|span");
		result = getHtmlByFilter(map,notMap,html);
		System.out.println("=============================result============================");
		System.out.println(result);
		System.out.println("==========================================================");
		System.out.println(getUrlAndTitleMap(result,"u/cms/www/201111/"));
	}
	/**
	 * 测试取得内容
	 * @throws ParserException
	 */
	public void testFetchContent() throws ParserException{
		String html = openFile("F:\\108784.shtml","gb2312");
		Map<String, String> map = new HashMap<String, String>();
		map.put("class", "content.*");
		Map<String, String> notMap = new HashMap<String, String>();
//		notMap.put(SINGLE_TAG, "style|script");
//		notMap.put("type", "text/javascript");
//		notMap.put("class", "icon_fx|blkComment otherContent_01");
//		notMap.put("style", "text-align: right;padding-right:10px;|margin-top:6px;|font-size: 12px ! important;|font-size:12px");
//		notMap.put("id", "fxwb|fxMSN|fxMSN|comment_t_show_top");
		html = getHtmlByFilter(map,notMap,html);
		//html = getPlainText(html);
		//System.out.println(html);
	}
	/**
	 * 测试取得图片
	 * @throws ParserException
	 */
	public void testReplaceHtmlImages() throws ParserException{
		String html = openFile("F:\\2006718059316482.htm","GB2312");
		Map<String, String> map = new HashMap<String, String>();
		map.put("class", "vclt-con");
		Map<String, String> notMap = new HashMap<String, String>();
		html = getHtmlByFilter(map,notMap,html);
		System.out.println("===================内容=================");
		System.out.println(html);
		Map<String, Object> resultMap = new HashMap<String, Object>();
		replaceHtmlResource(html,"u/cms/www/201111/",resultMap);
		System.out.println("===================图片内容=================");
		System.out.println(""+resultMap.get(Constants.RES_HTML_KEY));
	}
	/**
	 * 测试取得图片
	 * @throws ParserException
	 */
	public void testGetHtmlImages() throws ParserException{
		String html = openFile("F:\\temp\\181926.htm","GB2312");
//		Parser parser = new Parser();
//		parser.setInputHTML(html);
//		NodeFilter filter = new NodeClassFilter(RemarkNode.class);
//        NodeList nodes = parser.extractAllNodesThatMatch(filter);
//        for (int i = 0; i < nodes.size(); i++) {
//			Node textnode = (Node) nodes.elementAt(i);
//			System.out.println(textnode.toHtml());
//		}
		Map<String, String> map = new HashMap<String, String>();
		//map.put("id", "photoView");
		//map.put("id", "header");
		map.put("class", "ad-thumb-list");
		Map<String, String> notMap = new HashMap<String, String>();
		html = getHtmlByFilter(map,notMap,html);
		Map<String, Object> resultMap = new HashMap<String, Object>();
		getHtmlResource(html,resultMap,"");
		System.out.println("===================图片内容=================");
		System.out.println(resultMap.toString());
	}
	public void testGetPaginationList() throws ParserException{
		String html = openFile("F:\\temp\\181926.htm","GB2312");
//		Parser parser = new Parser();
//		parser.setInputHTML(html);
//		NodeFilter filter = new NodeClassFilter(RemarkNode.class);
//        NodeList nodes = parser.extractAllNodesThatMatch(filter);
//        for (int i = 0; i < nodes.size(); i++) {
//			Node textnode = (Node) nodes.elementAt(i);
//			System.out.println(textnode.toHtml());
//		}
		Map<String, String> map = new HashMap<String, String>();
		//map.put("id", "photoView");
		//map.put("id", "header");
		map.put("class", "ad-thumb-list");
		Map<String, String> notMap = new HashMap<String, String>();
		html = getHtmlByFilter(map,notMap,html);
		System.out.println("===================html=================");
		System.out.println(html);
		List<String> urlList = getUrlOrTitleListByType(html,Constants.LINK_KEY);
		System.out.println("===================连接=================");
		for(String url : urlList){
			System.out.println(url);
		}
	}
	
	/**
	 * 测试去掉内容中的超链接
	 * @throws ParserException
	 */
	public void testRemoveHtmlLink() throws ParserException{
		String html = openFile("F:\\2006718059316482.htm","GB2312");
		Map<String, String> map = new HashMap<String, String>();
		map.put("class", "vclt-con");
		Map<String, String> notMap = new HashMap<String, String>();
		html = getHtmlByFilter(map,notMap,html);
		System.out.println("===================内容=================");
		System.out.println(html);
		html = replaceHtmlLink(html);
		System.out.println("===================新内容=================");
		System.out.println(html);
	}
	/**
	 * 测试解析参数
	 */
	public void testParseParam(){
		Map<String, String> map = new HashMap<String, String>();
		populateParamMap(map,"class=articleList|tips,p,div");
		String tempKey = null;
		String tempValue = null;
		String[] tempValueArr = null;
		for (Iterator<String> it = map.keySet().iterator(); it.hasNext();) {
			tempKey = it.next();
			tempValue = map.get(tempKey);
			if(tempValue.contains("|")){
				tempValueArr = tempValue.split("\\|");
			}else{
				tempValueArr = new String[]{tempValue};
			}
			for(String value : tempValueArr){
				System.out.println("tempKey:" + tempKey);
				System.out.println("value:" + value);
			}
		}
	}
	/**
	 * 测试过滤标签
	 * @throws ParserException
	 */
	public void testRemarkFilter() throws ParserException{
		String html = openFile("F:\\6.shtml","GB2312");
		System.out.println("=========================过滤注释前HTML==================================");
		System.out.println(html);
		NodeFilter filter = new NodeClassFilter(RemarkNode.class);
		html = removeHtmlByFilter(new Parser(), filter, html);
		System.out.println("=========================过滤注释后HTML==================================");
		System.out.println(html);
	}
	/**
	 * 测试过滤标签
	 * @throws ParserException
	 */
	public static void testGetFieldValues() throws ParserException{
		String html = openFile("F:\\temp\\泡菜的做法.html","UTF-8");
		System.out.println("=========================过滤注释前HTML==================================");
		System.out.println(html);
		List<Filter> filters = new CopyOnWriteArrayList<Filter>();
		filters.add(new FieldFilter("text","class=tl_text",""));
		filters.add(new FieldFilter("text2","class=tl_text",""));
		//filters.add(new ContentAreaFilter("id=artibody","id=fxwb|fxMSN|fxMSN|comment_t_show_top"));
		HtmlParserImpl parseHtmlTool = new HtmlParserImpl(filters);
		
		System.out.println("=========================过滤注释后HTML==================================");
		System.out.println(JSONArray.fromObject(parseHtmlTool.getFieldValues(html)).toString());
		System.out.println(parseHtmlTool.getFieldValues(html));
	}
	
	/**
	 * 测试取得指定属性，指定属性值正则表达式匹配的所有CompositeTa
	 * @throws ParserException
	 */
	public void testGetCompList() throws ParserException{
		String html = openFile("F:\\108784.shtml","gb2312");
		System.out.println("=========================过滤注释前HTML==================================");
		//System.out.println(html);
//		Parser parser = new Parser();
//		parser.setInputHTML(html);
//        RegexFilter filter = new RegexFilter("content *");  
//        NodeList nodes = parser.extractAllNodesThatMatch(filter);
//        for (int i = 0; i < nodes.size(); i++) {
//			Node textnode = (Node) nodes.elementAt(i);
//			System.out.println(textnode.toHtml());
//		}
		Parser parser = new Parser();
		parser.setInputHTML(html);
		List<CompositeTag> tagList = getCompList(parser,"class","content.*");
		for(CompositeTag tag : tagList){
			System.out.println("getTagName : " + tag.toHtml()+"    getAttribute : "+tag.getAttribute("class"));
		}
//		Pattern pt = Pattern.compile("(content.*)");
//
//		Matcher m = pt.matcher(html);
//        String link = null;
//		while (m.find()) {
//			link = m.group(1);
//			System.out.println("url : " + link);
//			
//		}
	}
	/**
	 * 取得指定属性，指定属性值正则表达式匹配的所有CompositeTa
	 * @param parser Parser对象
	 * @param attributeName 属性
	 * @param attributeValueMatches 属性值正则表达式
	 * @return List<CompositeTag>
	 * @throws ParserException
	 */
	private List<CompositeTag> getCompList(Parser parser,String attributeName,String attributeValueMatches)
			throws ParserException {
		NodeFilter filter = new NodeClassFilter(BodyTag.class);
		NodeList nodes = parser.extractAllNodesThatMatch(filter);
		BodyTag bodyTag = (BodyTag)nodes.elementAt(0);
		List<CompositeTag> tagList = new CopyOnWriteArrayList<CompositeTag>();
		for(Node node : bodyTag.getChildrenAsNodeArray()){
			if(null != node.getChildren()){
				findAllCompositeTagByMatches(node,tagList,attributeName,attributeValueMatches);
			}
		}
		return tagList;
	}
	
	public static void main(String[] args) throws ParserException,
			URISyntaxException, IOException {
		List<Filter> filters = new CopyOnWriteArrayList<Filter>();
		//filters.add(new LinkAreaFilter("id=artibody","id=fxwb|fxMSN|fxMSN|comment_t_show_top"));
		//filters.add(new ContentAreaFilter("id=artibody","id=fxwb|fxMSN|fxMSN|comment_t_show_top"));
		HtmlParserImpl parseHtmlTool = new HtmlParserImpl(filters);
		//parseHtmlTool.testParseParam();
		//parseHtmlTool.testFetchLinkAndTitle();
		//parseHtmlTool.testFetchContent();
		//parseHtmlTool.testRemarkFilter();
		//parseHtmlTool.testReplaceHtmlImages();
		//parseHtmlTool.testRemoveHtmlLink();
		parseHtmlTool.testGetPaginationList();
		//parseHtmlTool.testGetCompList();
		//testGetFieldValues();
	}

}
